/*
 * tbinrealop.C - 'template' for a real binary operator, included for "/"
 */

void OPFNAME(Tobject& c, const Tobject& a, const Tobject& b) {
	int i,N;
	Treal X; Tcomplex Z;
	switch (pair(a.kind(),b.kind())) {
	case pair(Kint,Kint):
		c = OP(Treal(a.i),Treal(b.i));
		gl_nops++;
		break;
	case pair(Kint,Kreal):
		c = OP(Treal(a.i),b.x);
		gl_nops++;
		break;
	case pair(Kint,Kcomplex):
		c = OP(Tcomplex(a.i),b.z);
		gl_nops++;
		break;
	case pair(Kint,KIntArray):
		{
			X = Treal(a.i);
			N = b.dims().length();
			c.copydimsRealArray(b);
			Tint *bp = b.IntPtr();
			if (&b == &c) {
				Tobject c1;
				Treal *c1p = c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(X,Treal(bp[i]));
				c = c1;
			} else {
				Treal *cp = c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(X,Treal(bp[i]));
			}
			gl_nops+= N;
		}
		break;
	case pair(Kint,KRealArray):
		{
			c.copydimsRealArray(b);
			X = Treal(a.i);
			N = b.dims().length();
			Treal *cp=c.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(X,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kint,KComplexArray):
		{
			c.copydimsComplexArray(b);
			Z = Tcomplex(Treal(a.i),0.0);
			N = b.dims().length();
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kreal,Kint):
		c = OP(a.x,Treal(b.i));
		gl_nops++;
		break;
	case pair(Kreal,Kreal):
		c = OP(a.x,b.x);
		gl_nops++;
		break;
	case pair(Kreal,Kcomplex):
		c = OP(Tcomplex(a.x),b.z);
		gl_nops++;
		break;
	case pair(Kreal,KIntArray):
		{
			N = b.dims().length();
			Tint *bp=b.IntPtr();
			if (&c == &b) {
				Tobject c1;
				c1.copydimsRealArray(b);
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(a.x,bp[i]);
				c = c1;
			} else {
				c.copydimsRealArray(b);	// make c a real array of the same dims as b
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(a.x,bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(Kreal,KRealArray):
		{
			c.copydimsRealArray(b);
			N = b.dims().length();
			Treal *cp=c.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(a.x,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kreal,KComplexArray):
		{
			c.copydimsComplexArray(b);
			Z = a.x;
			N = b.dims().length();
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(Z,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(Kcomplex,Kint):
		c = OP(a.z,Tcomplex(b.i));
		gl_nops++;
		break;
	case pair(Kcomplex,Kreal):
		c = OP(a.z,Tcomplex(b.x));
		gl_nops++;
		break;
	case pair(Kcomplex,Kcomplex):
		c = OP(a.z,b.z);
		gl_nops++;
		break;
	case pair(Kcomplex,KIntArray):
		{
			N = b.dims().length();
			Tint *bp=b.IntPtr();
			if (&c == &b) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(a.z,bp[i]);
				c = c1;
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(a.z,bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(Kcomplex,KRealArray):
		{
			N = b.dims().length();
			Treal *bp=b.RealPtr();
			if (&c == &b) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(a.z,bp[i]);
				c = c1;
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(a.z,bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(Kcomplex,KComplexArray):
		{
			c.copydimsComplexArray(b);
			N = b.dims().length();
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(a.z,bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kint):
		{
			c.copydimsRealArray(a);
			X = Treal(b.i);
			N = a.dims().length();
			Tint *ap=a.IntPtr();
			if (&a == &c) {
				Tobject c1;
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],X);
				c = c1;
			} else {
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			}
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kreal):
		{
			N = a.dims().length();
			Tint *ap=a.IntPtr();
			if (&c == &a) {
				Tobject c1;
				c1.copydimsRealArray(a);
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],b.x);
				c = c1;
			} else {
				c.copydimsRealArray(a);
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],b.x);
			}
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,Kcomplex):
		{
			N = a.dims().length();
			Tint *ap=a.IntPtr();
			if (&c == &a) {
				Tobject c1;
				c1.copydimsComplexArray(a);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],b.z);
				c = c1;
			} else {
				c.copydimsComplexArray(a);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],b.z);
			}
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KIntArray):
		N = a.dims().length();
		if (N != b.dims().length()) LengthError(OPCHAR,a,b);
		if (&c == &a || &c == &b) {
			Tobject c1;
			c1.copydimsRealArray(a);
			Treal *c1p=c1.RealPtr(); Tint *ap=a.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) c1p[i] = OP(Treal(ap[i]),Treal(bp[i]));
			c = c1;
		} else {
			c.copydimsRealArray(a);
			Treal *cp=c.RealPtr(); Tint *ap=a.IntPtr(), *bp=b.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(Treal(ap[i]),Treal(bp[i]));
		}
		gl_nops+= N;
		break;
	case pair(KIntArray,KRealArray):
		{
			c.copydimsRealArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Treal *cp=c.RealPtr(), *bp=b.RealPtr(); Tint *ap=a.IntPtr();
			for (i=0; i<N; i++) cp[i] = OP(Treal(ap[i]),bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KIntArray,KComplexArray):
		{
			c.copydimsComplexArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *ap=a.IntPtr(); Tcomplex *bp=b.ComplexPtr();
			if (&a == &c) {
				Tobject c1;
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				c = c1;
			} else {
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kint):
		{
			c.copydimsRealArray(a);
			X = Treal(b.i);
			N = a.dims().length();
			Treal *cp=c.RealPtr(), *ap=a.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],X);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kreal):
		{
			c.copydimsRealArray(a);
			N = a.dims().length();
			Treal *cp=c.RealPtr(), *ap=a.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],b.x);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,Kcomplex):
		{
			N = a.dims().length();
			Treal *ap=a.RealPtr();
			if (&c == &a) {
				Tobject c1;
				c1.copydimsComplexArray(a);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],b.z);
				c = c1;
			} else {
				c.copydimsComplexArray(a);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],b.z);
			}
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KIntArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tint *bp=b.IntPtr();
			Treal *ap=a.RealPtr();
			if (&b == &c) {
				Tobject c1;
				c1.copydimsRealArray(a);
				Treal *c1p=c1.RealPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],Treal(bp[i]));
				c = c1;
			} else {
				c.copydimsRealArray(a);
				Treal *cp=c.RealPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],Treal(bp[i]));
			}
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KRealArray):
		{
			c.copydimsRealArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Treal *cp=c.RealPtr(), *ap=a.RealPtr(), *bp=b.RealPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			gl_nops+= N;
		}
		break;
	case pair(KRealArray,KComplexArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Treal *ap=a.RealPtr(); Tcomplex *bp=b.ComplexPtr();
			if (&a == &c) {
				Tobject c1;
				c1.copydimsComplexArray(b);
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
			} else {
				c.copydimsComplexArray(b);
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kint):
		{
			c.copydimsComplexArray(a);
			N = a.dims().length();
			Z = Tcomplex(b.i);
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kreal):
		{
			c.copydimsComplexArray(a);
			N = a.dims().length();
			Z = Tcomplex(b.x);
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],Z);
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,Kcomplex):
		{
			c.copydimsComplexArray(a);
			N = a.dims().length();
			Tcomplex *cp=c.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],b.z);
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KIntArray):
		{
			c.copydimsComplexArray(a);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tcomplex *ap=a.ComplexPtr(); Tint *bp=b.IntPtr();
			if (&b == &c) {
				Tobject c1;
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				c = c1;
			} else {
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KRealArray):
		{
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tcomplex *ap=a.ComplexPtr(); Treal *bp=b.RealPtr();
			c.copydimsComplexArray(a);
			if (&b == &c) {
				Tobject c1;
				Tcomplex *c1p=c1.ComplexPtr();
				for (i=0; i<N; i++) c1p[i] = OP(ap[i],bp[i]);
				c = c1;
			} else {
				Tcomplex *cp=c.ComplexPtr();
				for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			}
			gl_nops+= N;
		}
		break;
	case pair(KComplexArray,KComplexArray):
		{
			c.copydimsComplexArray(b);
			N = a.dims().length();
			if (N != b.dims().length()) LengthError(OPCHAR,a,b);
			Tcomplex *cp=c.ComplexPtr(), *bp=b.ComplexPtr(), *ap=a.ComplexPtr();
			for (i=0; i<N; i++) cp[i] = OP(ap[i],bp[i]);
			gl_nops+= N;
		}
		break;
	default:
		BinError(OPCHAR,a,b);
		break;
	}
}
